import { Zone } from "@prisma/client";
import { LoadedZoneEntity } from "./entities/LoadedZoneEntity";
import ZoneRepository from "./repositories/zone";

class ZoneManager {
    private static _instance: ZoneManager;
    private _loadedZones: LoadedZoneEntity[] = [];

    private constructor() {}

    // Singleton instance getter
    public static getInstance(): ZoneManager {
        if (!ZoneManager._instance) {
            ZoneManager._instance = new ZoneManager();
        }
        return ZoneManager._instance;
    }

    // Method to initialize zone loading
    public async boot() {
        const zoneRepository = new ZoneRepository();
        const zones = await zoneRepository.getAll();

        for (const zone of zones) {
            this.loadZone(zone);
        }

        console.log('[✅] ZoneManager loaded');
    }

    // Method to handle individual zone loading
    public loadZone(zone: Zone) {
        const loadedZone = new LoadedZoneEntity(zone);
        this._loadedZones.push(loadedZone);
        console.log(`[✅] Zone ID ${zone.id} loaded`);
    }

    // Method to handle individual zone unloading
    public unloadZone(zoneId: number) {
        const index = this._loadedZones.findIndex(loadedZone => loadedZone.getZone().id === zoneId);
        if (index > -1) {
            this._loadedZones.splice(index, 1);
            console.log(`[❌] Zone ID ${zoneId} unloaded`);
        }
    }

    // Getter for loaded zones
    public getLoadedZones(): LoadedZoneEntity[] {
        return this._loadedZones;
    }
}

export default ZoneManager;
